package com.example.zdrojaknotifier;

import java.util.Date;

import android.content.ContentProvider;
import android.content.ContentUris;
import android.content.ContentValues;
import android.content.Context;
import android.content.Intent;
import android.content.UriMatcher;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;
import android.net.Uri;

public class ZdrojakProvider extends ContentProvider {

	private static final int ARTICLES = 0;
	private static final int ARTICLE_ID = 1;

	private static final String[] MANDATORY_COLUMNS = { ZdrojakContract.TITLE,
			ZdrojakContract.LINK, ZdrojakContract.DESCRIPTION,
			ZdrojakContract.AUTHOR, ZdrojakContract.PUBDATE };

	private static final UriMatcher uriMatcher = new UriMatcher(
			UriMatcher.NO_MATCH);

	static {
		uriMatcher.addURI(ZdrojakContract.AUTHORITY, ZdrojakContract.ARTICLES,
				ARTICLES);
		uriMatcher.addURI(ZdrojakContract.AUTHORITY, ZdrojakContract.ARTICLES
				+ "/#", ARTICLE_ID);
	}

	protected static final String DATABASE_NAME = "zdrojak";
	protected static final int DATABASE_VERSION = 2;

	protected static final String TB_NAME = "articles";

	private SQLiteOpenHelper openHelper;

	static class DatabaseHelper extends SQLiteOpenHelper {

		DatabaseHelper(Context context) {
			super(context, DATABASE_NAME, null, DATABASE_VERSION);
		}

		@Override
		public void onCreate(SQLiteDatabase db) {
			db.execSQL("CREATE TABLE " + TB_NAME + " (" + ZdrojakContract._ID
					+ " INTEGER PRIMARY KEY," + ZdrojakContract.TITLE
					+ " TEXT NOT NULL," + ZdrojakContract.LINK
					+ " TEXT NOT NULL," + ZdrojakContract.DESCRIPTION
					+ " TEXT NOT NULL," + ZdrojakContract.AUTHOR
					+ " TEXT NOT NULL," + ZdrojakContract.PUBDATE
					+ " INTEGER NOT NULL," + ZdrojakContract.VIEWED
					+ " BOOLEAN NOT NULL DEFAULT 0, " + ZdrojakContract.ADDED
					+ " INTEGER NOT NULL" + ");");
		}

		/*
		 * Bylo by lepší vytvořit pro každou změnu struktury databáze nějaký
		 * upgradovací nedestruktivní SQL příkaz, ačkoli tady ztráta dat tolik
		 * nebolí.
		 */
		@Override
		public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {
			db.execSQL("DROP TABLE IF EXISTS " + TB_NAME);
			onCreate(db);
		}
	}

	@Override
	public int delete(Uri uri, String selection, String[] selectionArgs) {
		SQLiteDatabase db = openHelper.getWritableDatabase();

		int count;

		switch (uriMatcher.match(uri)) {
		case ARTICLES:
			count = db.delete(TB_NAME, selection, selectionArgs);
			break;

		case ARTICLE_ID:
			// Musíme vytvořít podmínku WHERE pro dané ID, ale zároveň musíme
			// zachovat případné where předané uživatelem.
			String[] newArgs = createSelectionArgsWithId(selectionArgs, uri
					.getPathSegments().get(1));
			String where = createSelectionWithId(selection);

			count = db.delete(TB_NAME, where, newArgs);
			break;

		default:
			throw new IllegalArgumentException("Unknown URI " + uri);
		}

		// Vrátíme počet smazaných řádků
		return count;
	}

	@Override
	public Uri insert(Uri uri, ContentValues values) {
		// Neměl by se používat sám o sobě - nekontroluje, jestli tam článek už
		// není a také neobnovuje notifikaci.

		if (uriMatcher.match(uri) != ARTICLES) {
			throw new IllegalArgumentException("Unknown URI " + uri);
		}

		checkMandatoryColumns(values);

		if (!values.containsKey(ZdrojakContract.VIEWED))
			values.put(ZdrojakContract.VIEWED, false);

		values.put(ZdrojakContract.ADDED, new Date().getTime());

		SQLiteDatabase db = openHelper.getWritableDatabase();

		long id = db.insert(TB_NAME, null, values);

		if (id > 0) {
			Uri noteUri = ContentUris.withAppendedId(
					ZdrojakContract.CONTENT_URI_ARTICLES, id);
			getContext().getContentResolver().notifyChange(noteUri, null);
			return noteUri;
		} else {
			return null;
		}
	}

	@Override
	public int update(Uri uri, ContentValues values, String selection,
			String[] selectionArgs) {
		SQLiteDatabase db = openHelper.getWritableDatabase();

		int count;

		switch (uriMatcher.match(uri)) {
		case ARTICLES:
			count = db.update(TB_NAME, values, selection, selectionArgs);
			break;

		case ARTICLE_ID:
			String[] newArgs = createSelectionArgsWithId(selectionArgs, uri
					.getPathSegments().get(1));
			String where = createSelectionWithId(selection);

			count = db.update(TB_NAME, values, where, newArgs);
			break;

		default:
			throw new IllegalArgumentException("Unknown URI " + uri);
		}

		sendUpdateBroadcast();
		getContext().getContentResolver().notifyChange(uri, null);

		return count;
	}

	@Override
	public String getType(Uri uri) {

		switch (uriMatcher.match(uri)) {
		case ARTICLES:
			return ZdrojakContract.MIME_TYPE_DIR;

		case ARTICLE_ID:
			return ZdrojakContract.MIME_TYPE_ITEM;

		default:
			throw new IllegalArgumentException("Unknown URI " + uri);
		}
	}

	@Override
	public int bulkInsert(Uri uri, ContentValues[] values) {
		if (uriMatcher.match(uri) != ARTICLES) {
			throw new IllegalArgumentException("Unknown URI " + uri);
		}

		int insertCount = 0;
		long maxDate = getMaxDate();

		for (ContentValues v : values) {
			checkMandatoryColumns(v);
			long date = v.getAsLong(ZdrojakContract.PUBDATE);

			if (date > maxDate) {
				if (insert(uri, v) != null)
					insertCount++;
			}
		}

		sendUpdateBroadcast();

		return insertCount;
	}

	private void checkMandatoryColumns(ContentValues values) {
		for (String key : MANDATORY_COLUMNS) {
			if (!values.containsKey(key)) {
				throw new NullPointerException(
						"ContentValues must contain key " + key);
			}
		}
	}

	@Override
	public boolean onCreate() {
		openHelper = new DatabaseHelper(getContext());
		return true;
	}

	@Override
	public Cursor query(Uri uri, String[] projection, String selection,
			String[] selectionArgs, String sortOrder) {
		SQLiteDatabase db = openHelper.getReadableDatabase();

		if (sortOrder == null || sortOrder.equals(""))
			sortOrder = ZdrojakContract.DEFAULT_SORT_ORDER;

		switch (uriMatcher.match(uri)) {
		case ARTICLES:
			break;

		case ARTICLE_ID:
			selectionArgs = createSelectionArgsWithId(selectionArgs, uri
					.getPathSegments().get(1));
			selection = createSelectionWithId(selection);
			break;

		default:
			throw new IllegalArgumentException("Unknown URI " + uri);
		}

		Cursor c = db.query(TB_NAME, projection, selection, selectionArgs,
				null, null, sortOrder);

		c.setNotificationUri(getContext().getContentResolver(), uri);

		return c;
	}

	private long getMaxDate() {
		SQLiteDatabase db = openHelper.getReadableDatabase();

		String[] cols = { ZdrojakContract.PUBDATE };

		Cursor c = db.query(TB_NAME, cols, null, null, null, null,
				ZdrojakContract.SORT_ORDER_PUBDATE, "1");

		if (!c.moveToFirst())
			return -1;

		else
			return c.getLong(0);
	}

	private String[] createSelectionArgsWithId(String[] oldArgs, String id) {

		String[] newArgs;
		if (oldArgs != null)
			newArgs = copyOfArray(oldArgs, oldArgs.length + 1);
		else
			newArgs = new String[1];

		newArgs[newArgs.length - 1] = id;

		return newArgs;
	}

	// Pro zjednodušení jen pro String[] a neřeším genericitu
	private static String[] copyOfArray(String[] old, int newLength) {
		String[] newArray = new String[newLength];

		int oldLength = old.length;

		for (int i = 0; i < Math.min(oldLength, newLength); i++) {
			if (i < oldLength)
				newArray[i] = old[i];
			else
				newArray[i] = null;
		}

		return newArray;
	}

	private String createSelectionWithId(String oldSelection) {
		String where = ZdrojakContract._ID + " = ?";

		if (oldSelection != null && !oldSelection.equals("")) {
			where = oldSelection + " AND " + where;
		}

		return where;
	}

	private void sendUpdateBroadcast() {
		Intent i = new Intent(ZdrojakContract.ACTION_ZDROJAK_UPDATED);
		getContext().sendBroadcast(i);
	}
}
